package ru.ldeloff.servermonitorbot.repository;

import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import ru.ldeloff.servermonitorbot.config.SshConfig;
import ru.ldeloff.servermonitorbot.model.SshServer;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Slf4j
@Repository
public class SshRepositoryImpl implements SshRepository {

    private final List<SshServer> sshServers;

    @Autowired
    public SshRepositoryImpl(SshConfig sshConfig) {
        this.sshServers = sshConfig.getServers();
    }

    private List<Session> sessions = new ArrayList<>();

    @Override
    public void connectToAllServer() {
        sshServers.forEach(sshServer -> {
            try {
                Session session = connectToServer(sshServer);
                sshServer.setSession(session);
                log.info("Успешно подключён к " + sshServer.getHost());
                sessions.add(session);
            } catch (JSchException e) {
                log.warn("Не удалось соединиться с " + sshServer.getHost() + ": " + e.getMessage());
            }
        });
    }

    @Override
    public Session connectToServer(SshServer sshServer) throws JSchException {
        Session session = new JSch().getSession(sshServer.getUser(),
                sshServer.getHost(),
                sshServer.getPort());
        session.setPassword(sshServer.getPassword());
        session.setConfig("StrictHostKeyChecking", "no");
        session.connect();
        return session;
    }

    @Override
    public void disconnectSessions() {
        if (!sessions.isEmpty()) {
            sessions.forEach(Session::disconnect);
            sessions.forEach(sessions::remove);
        }
    }

    @Override
    public String getStatusSessions() {
        StringBuilder text = new StringBuilder("Статус соединения: \n");
        sshServers.forEach(server -> {
            text.append(server.getName())
                    .append(": ")
                    .append(checkStatusServer(server))
                    .append("\n");
        });
        return String.valueOf(text);
    }

    @Override
    public List<SshServer> getSshServers() {
        return sshServers;
    }

    private String checkStatusServer(SshServer server) {
        if (Objects.isNull(server.getSession())) {
            return "нет соединения";
        } else {
            return server.getSession().isConnected() ? "OK" : "отключён";
        }
    }

}
